package weaver.formmode.hty.modeexpand.activity;

import com.alibaba.fastjson.JSON;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import weaver.common.DateUtil;
import weaver.conn.RecordSet;
import weaver.formmode.customjavacode.AbstractModeExpandJavaCodeNew;
import weaver.general.Util;
import weaver.hrm.User;
import weaver.soa.workflow.request.*;
import weaver.workflow.webservices.WorkflowDetailTableInfo;
import weaver.workflow.webservices.WorkflowMainTableInfo;
import weaver.workflow.webservices.WorkflowRequestTableField;
import weaver.workflow.webservices.WorkflowRequestTableRecord;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static weaver.formmode.hty.modeexpand.activity.FlowUtil.*;

/**
 * @author:dxfeng
 * @createTime: 2024/09/06
 * @version: 1.0
 */
public class InitiateRatingExpand extends AbstractModeExpandJavaCodeNew {
    private String FLOW_ID = "136";

    @Override
    public Map<String, String> doModeExpand(Map<String, Object> param) {
        Map<String, String> result = new HashMap<>(2);
        RecordSet rs = new RecordSet();
        try {
            int billId;
            int modeId;
            RequestInfo requestInfo = (RequestInfo) param.get("RequestInfo");
            if (requestInfo != null) {
                User user = (User) param.get("user");
                billId = Util.getIntValue(requestInfo.getRequestid());
                modeId = Util.getIntValue(requestInfo.getWorkflowid());
                if (billId > 0 && modeId > 0) {
                    Map<String, String> mainDataMap = new HashMap<>(16);
                    MainTableInfo mainTableInfo = requestInfo.getMainTableInfo();
                    Property[] properties = mainTableInfo.getProperty();
                    for (Property property : properties) {
                        mainDataMap.put(property.getName(), property.getValue());
                    }
                    List<Map<String, String>> detailList = new ArrayList<>();
                    DetailTableInfo detailTableInfo = requestInfo.getDetailTableInfo();
                    DetailTable detailTable = detailTableInfo.getDetailTable(0);
                    Row[] rows = detailTable.getRow();
                    for (Row row : rows) {
                        Map<String, String> detailDataMap = new HashMap<>();
                        detailDataMap.put("id", row.getId());
                        Cell[] cells = row.getCell();
                        for (Cell cell : cells) {
                            String name = cell.getName();
                            String value = cell.getValue();
                            detailDataMap.put(name, value);
                        }
                        detailList.add(detailDataMap);
                    }

                    // 遍历明细表数据、查询对应方案数据，创建流程
                    String khnd = mainDataMap.get("khnd");
                    String khjd = mainDataMap.get("khjd");
                    for (Map<String, String> detailItem : detailList) {
                        String activityDetailId = detailItem.get("id");
                        String khry = detailItem.get("khry");
                        matchPlanData(rs, user, activityDetailId, khnd, khjd, khry,String.valueOf(billId), FlowUtil.parseBlankToNull(mainDataMap.get("zpbz")),FlowUtil.parseBlankToNull(mainDataMap.get("zjsjqz")), FlowUtil.parseBlankToNull(mainDataMap.get("xxsjqz")));
                    }

                }
            }
        } catch (Exception e) {
            rs.writeLog(e);
            result.put("errmsg", "添加评价人员拓展异常");
            result.put("flag", "false");
        }
        return result;
    }

    // 遍历数据创建流程


    /**
     * @param rs
     * @param khnd
     * @param khjd
     */
    private void matchPlanData(RecordSet rs, User user, String activityDetailId, String khnd, String khjd, String khry,String id,String zpbz,String zjsjqz,String xxsjqz) {
        rs.executeQuery("select * from uf_khfahty where zrr = ? and nd = ? and jd = ? And formmodeid = 312", khry, khnd, khjd);
        Map<String, String> mainDataMap = new HashMap<>();
        String mainId = "";
        if (rs.next()) {
            mainId = rs.getString("id");
            String[] columnNames = rs.getColumnName();
            for (String columnName : columnNames) {
                mainDataMap.put(columnName.toLowerCase(), FlowUtil.parseBlankToNull(rs.getString(columnName)));
            }
        }

        // 未查到数据，或者主表ID，跳过该数据
        if (StringUtils.isBlank(mainId) || mainDataMap.isEmpty()) {
            rs.writeLog("未匹配到方案数据，khnd==" + khnd + ",khjd==" + khjd + ",khry==" + khry);
            return;
        }
        rs.writeLog("mainDataMap==="+ JSON.toJSONString(mainDataMap));

        // 构建主表流程数据
        WorkflowRequestTableField[] workflowRequestTableField = {
                createWorkflowRequestTableField("zrr", FlowUtil.parseBlankToNull(mainDataMap.get("zrr"))),
                createWorkflowRequestTableField("gs", FlowUtil.parseBlankToNull(mainDataMap.get("gs"))),
                createWorkflowRequestTableField("bm", FlowUtil.parseBlankToNull(mainDataMap.get("bm"))),
                createWorkflowRequestTableField("gw", FlowUtil.parseBlankToNull(mainDataMap.get("gw"))),
                createWorkflowRequestTableField("zrrsj", FlowUtil.parseBlankToNull(mainDataMap.get("zrrsj"))),
                createWorkflowRequestTableField("zgld", FlowUtil.parseBlankToNull(mainDataMap.get("zgld"))),
                createWorkflowRequestTableField("zgldsj", FlowUtil.parseBlankToNull(mainDataMap.get("zgldsj"))),
                createWorkflowRequestTableField("xgld", FlowUtil.parseBlankToNull(mainDataMap.get("xgld"))),
                createWorkflowRequestTableField("xgldsj", FlowUtil.parseBlankToNull(mainDataMap.get("xgldsj"))),
                createWorkflowRequestTableField("khdf", FlowUtil.parseBlankToNull(mainDataMap.get("khdf"))),
                createWorkflowRequestTableField("khdj", FlowUtil.parseBlankToNull(mainDataMap.get("khdj"))),
                createWorkflowRequestTableField("kpiqz", FlowUtil.parseBlankToNull(mainDataMap.get("kpiqz"))),
                createWorkflowRequestTableField("zdgzqz", FlowUtil.parseBlankToNull(mainDataMap.get("zdgzqz"))),


                createWorkflowRequestTableField("khhd", id),
                createWorkflowRequestTableField("nd", khnd),
                createWorkflowRequestTableField("jd", khjd),
                createWorkflowRequestTableField("zpfzb", zpbz),
                createWorkflowRequestTableField("zgfzb", zjsjqz),
                createWorkflowRequestTableField("xgfzb", xxsjqz)
        };
        rs.writeLog("workflowRequestTableField=="+JSON.toJSONString(workflowRequestTableField));

        WorkflowMainTableInfo workflowMainTableInfo = new WorkflowMainTableInfo();
        WorkflowRequestTableRecord[] workflowRequestTableRecord = new WorkflowRequestTableRecord[1];
        workflowMainTableInfo.setRequestRecords(workflowRequestTableRecord);
        workflowRequestTableRecord[0] = new WorkflowRequestTableRecord();
        workflowRequestTableRecord[0].setWorkflowRequestTableFields(workflowRequestTableField);

        // 构建明细表数据
        WorkflowDetailTableInfo[] workflowDetailTableInfoArray = new WorkflowDetailTableInfo[3];
        workflowDetailTableInfoArray[0] = new WorkflowDetailTableInfo();
        workflowDetailTableInfoArray[1] = new WorkflowDetailTableInfo();
        workflowDetailTableInfoArray[2] = new WorkflowDetailTableInfo();

        // 明细表一
        rs.executeQuery("select * from uf_khfahty_dt1 where mainid = ?", mainId);
        List<Map<String, String>> detail1 = getRecordMapList(rs);
        if (CollectionUtils.isNotEmpty(detail1)) {
            WorkflowRequestTableRecord[] workflowRequestTableRecordDt = new WorkflowRequestTableRecord[detail1.size()];
            for (int i = 0; i < detail1.size(); i++) {
                Map<String, String> item = detail1.get(i);
                WorkflowRequestTableField[] workflowRequestTableFieldDt = {
                        createWorkflowRequestTableField("zblx", item.get("zblx")),
                        createWorkflowRequestTableField("xh", item.get("xh")),
                        createWorkflowRequestTableField("khzb", item.get("khzb")),
                        createWorkflowRequestTableField("tqwcz", item.get("tqwcz")),
                        createWorkflowRequestTableField("bqdxz", item.get("bqdxz")),
                        createWorkflowRequestTableField("bqdbz", item.get("bqdbz")),
                        createWorkflowRequestTableField("bqtzz", item.get("bqtzz")),
                        createWorkflowRequestTableField("pfqz", item.get("pfqz")),
                        createWorkflowRequestTableField("khbz", item.get("khbz")),
                        createWorkflowRequestTableField("sjly", item.get("sjly")),
                        createWorkflowRequestTableField("zpf", item.get("zpf")),
                        createWorkflowRequestTableField("zgf", item.get("zgf")),
                        createWorkflowRequestTableField("xgf", item.get("xgf")),
                        createWorkflowRequestTableField("hjf", item.get("hjf")),
                        createWorkflowRequestTableField("zbmc", item.get("zbmc"))
                };
                workflowRequestTableRecordDt[i] = new WorkflowRequestTableRecord();
                workflowRequestTableRecordDt[i].setWorkflowRequestTableFields(workflowRequestTableFieldDt);
                workflowDetailTableInfoArray[0].setWorkflowRequestTableRecords(workflowRequestTableRecordDt);
            }
        }


        // 明细表二
        rs.executeQuery("select * from uf_khfahty_dt2 where mainid = ?", mainId);
        List<Map<String, String>> detail2 = getRecordMapList(rs);
        if (CollectionUtils.isNotEmpty(detail2)) {
            WorkflowRequestTableRecord[] workflowRequestTableRecordDt = new WorkflowRequestTableRecord[detail1.size()];
            for (int i = 0; i < detail2.size(); i++) {
                Map<String, String> item = detail2.get(i);
                WorkflowRequestTableField[] workflowRequestTableFieldDt = {
                        createWorkflowRequestTableField("zblx", item.get("zblx")),
                        createWorkflowRequestTableField("xh", item.get("xh")),
                        createWorkflowRequestTableField("zdgzrwmb", item.get("zdgzrwmb")),
                        createWorkflowRequestTableField("pfqz", item.get("pfqz")),
                        createWorkflowRequestTableField("pjbz", item.get("pjbz")),
                        createWorkflowRequestTableField("sjly", item.get("sjly")),
                        createWorkflowRequestTableField("zpf", item.get("zpf")),
                        createWorkflowRequestTableField("zgf", item.get("zgf")),
                        createWorkflowRequestTableField("xgf", item.get("xgf")),
                        createWorkflowRequestTableField("hjf", item.get("hjf"))
                };
                workflowRequestTableRecordDt[i] = new WorkflowRequestTableRecord();
                workflowRequestTableRecordDt[i].setWorkflowRequestTableFields(workflowRequestTableFieldDt);
                workflowDetailTableInfoArray[1].setWorkflowRequestTableRecords(workflowRequestTableRecordDt);
            }
        }

        // 明细表三
        rs.executeQuery("select * from uf_khfahty_dt3 where mainid = ?", mainId);
        List<Map<String, String>> detail3 = getRecordMapList(rs);
        if (CollectionUtils.isNotEmpty(detail3)) {
            WorkflowRequestTableRecord[] workflowRequestTableRecordDt = new WorkflowRequestTableRecord[detail1.size()];
            for (int i = 0; i < detail3.size(); i++) {
                Map<String, String> item = detail3.get(i);
                WorkflowRequestTableField[] workflowRequestTableFieldDt = {
                        createWorkflowRequestTableField("zblx", item.get("zblx")),
                        createWorkflowRequestTableField("xh", item.get("xh")),
                        createWorkflowRequestTableField("sxmc", item.get("sxmc")),
                        createWorkflowRequestTableField("pjbz", item.get("pjbz")),
                        createWorkflowRequestTableField("sjly", item.get("sjly")),
                        createWorkflowRequestTableField("zpf", item.get("zpf")),
                        createWorkflowRequestTableField("zgf", item.get("zgf")),
                        createWorkflowRequestTableField("xgf", item.get("xgf")),
                        createWorkflowRequestTableField("hjf", item.get("hjf")),
                };
                workflowRequestTableRecordDt[i] = new WorkflowRequestTableRecord();
                workflowRequestTableRecordDt[i].setWorkflowRequestTableFields(workflowRequestTableFieldDt);
                workflowDetailTableInfoArray[2].setWorkflowRequestTableRecords(workflowRequestTableRecordDt);
            }
        }

        String workflow = createWorkflow(FLOW_ID, "考核评分-" + user.getLastname() + "-" + DateUtil.getCurrentDate(), user.getUID() + "", "0", workflowMainTableInfo, workflowDetailTableInfoArray);
        // 更新建模明细表流程ID
        rs.executeUpdate("update uf_htyjxhd_dt1 set jxpflc = ? where id = ?", workflow, activityDetailId);

    }
}
