package weaver.formmode.hty.modeexpand.performance;

import cn.hutool.core.convert.Convert;
import com.alibaba.fastjson.JSON;
import weaver.conn.RecordSet;
import weaver.formmode.customjavacode.AbstractModeExpandJavaCodeNew;
import weaver.soa.workflow.request.*;

import java.util.*;

/**
 * @author:dxfeng
 * @createTime: 2024/09/07
 * @version: 1.0
 */
public class RegistrationDistributionExpand extends AbstractModeExpandJavaCodeNew {
    @Override
    public Map<String, String> doModeExpand(Map<String, Object> param) {
        Map<String, String> resultMap = new HashMap<>(2);
        RecordSet rs = new RecordSet();
        try {
            int billId;
            int modeId;
            RequestInfo requestInfo = (RequestInfo) param.get("RequestInfo");
            if (requestInfo != null) {
                {

                    DetailTableInfo detailTableInfo = requestInfo.getDetailTableInfo();
                    DetailTable detailTable0 = detailTableInfo.getDetailTable(0);

                    List<Rule> ruleList = new ArrayList<>();
                    Row[] rows0 = detailTable0.getRow();
                    for (Row row : rows0) {
                        Rule rule = new Rule();
                        Cell[] cells = row.getCell();
                        for (Cell cell : cells) {
                            String name = cell.getName();
                            String value = cell.getValue();
                            if ("jxdj".equals(name)) {
                                rule.setRuleName(value);
                            } else if ("fbbl".equals(name)) {
                                rule.setRuleRange(Double.parseDouble(value));
                            }

                        }
                        ruleList.add(rule);
                    }

                    ruleList.forEach(item -> rs.writeLog(JSON.toJSONString(item)));


                    DetailTable detailTable1 = detailTableInfo.getDetailTable(1);
                    Row[] rows1 = detailTable1.getRow();
                    List<Result> resultList = new ArrayList<>();
                    for (Row row : rows1) {
                        Cell[] cells = row.getCell();
                        Result result = new Result();
                        result.setId(row.getId());
                        for (Cell cell : cells) {
                            String name = cell.getName();
                            String value = cell.getValue();
                            if ("khzzdf".equals(name)) {
                                result.setScore(Convert.toDouble(value, 0D));
                            }
                        }
                        resultList.add(result);
                    }

                    // 所有人员排名
                    LinkedHashSet<Integer> rankSet = calculateRankings(resultList);
                    rs.writeLog("排名");
                    resultList.forEach(item -> rs.writeLog(JSON.toJSONString(item)));

                    ArrayList<Integer> rankList = new ArrayList<>(rankSet);
                    // 根据排名，按照分部比例划分
                    int subStart = 0;
                    double currentRange = 0.0;
                    for (int i = 0; i < ruleList.size(); i++) {
                        Rule rule = ruleList.get(i);
                        double ruleRange = rule.getRuleRange() / 100;
                        currentRange += ruleRange;
                        rs.writeLog("ruleRange==" + ruleRange);
                        rs.writeLog("currentRange==" + currentRange);
                        //int subEnd = subStart!=0?(int) Math.floor(rankList.size() * ruleRange):(int) Math.ceil(rankList.size() * ruleRange);
                        // 向下取
                        int subEnd = (int) Math.floor(rankList.size() * currentRange);
                        if (i == 0 && subEnd < 1) {
                            subEnd = 1;
                        }
                        if (i == ruleList.size() && subEnd < ruleList.size()) {
                            subEnd = rankList.size();
                        }
                        List<Integer> rangeList = rankList.subList(subStart, subEnd);
                        for (Integer integer : rangeList) {
                            for (Result result : resultList) {
                                if (result.getRank().equals(integer)) {
                                    result.setLevel(rule.getRuleName());
                                }
                            }
                        }
                        subStart = subEnd;
                    }

                    rs.writeLog("划分等级");

                    // 更新明细表
                    String updateSql = "update uf_jxdjfb_dt2 set khdj = ? where id = ?";
                    for (Result result : resultList) {
                        rs.writeLog(JSON.toJSONString(result));
                        rs.executeUpdate(updateSql, result.getLevel(), result.getId());
                    }
                }
            }
        } catch (Exception e) {
            rs.writeLog(e);
            resultMap.put("errmsg", e.getMessage());
            resultMap.put("flag", "false");
        }
        return resultMap;
    }

    public static LinkedHashSet<Integer> calculateRankings(List<Result> resultList) {
        LinkedHashSet<Integer> rankSet = new LinkedHashSet<>();


        resultList.sort((o1, o2) -> {
            return o2.getScore().compareTo(o1.getScore()); // 按值（分数）降序排序
        });

        // 计算排名并输出
        int rank = 1;
        double prevScore = -1;
        for (Result result : resultList) {
            double score = result.getScore();
            if (score < prevScore) {
                rank++;
            }
            prevScore = score;
            result.setRank(rank);
            rankSet.add(rank);

        }

        return rankSet;
    }

    static class Rule {
        String ruleName;
        double ruleRange;

        public Rule() {

        }

        public Rule(String ruleName, double ruleRange) {
            this.ruleName = ruleName;
            this.ruleRange = ruleRange;
        }

        public String getRuleName() {
            return ruleName;
        }

        public void setRuleName(String ruleName) {
            this.ruleName = ruleName;
        }

        public double getRuleRange() {
            return ruleRange;
        }

        public void setRuleRange(double ruleRange) {
            this.ruleRange = ruleRange;
        }
    }

    static class Result {
        String id;
        Double score;
        String level;
        Integer rank;

        public String getId() {
            return id;
        }

        public void setId(String id) {
            this.id = id;
        }

        public Double getScore() {
            return score;
        }

        public void setScore(Double score) {
            this.score = score;
        }

        public String getLevel() {
            return level;
        }

        public void setLevel(String level) {
            this.level = level;
        }

        public Integer getRank() {
            return rank;
        }

        public void setRank(Integer rank) {
            this.rank = rank;
        }
    }
}
