package weaver.formmode.poc.modeexpand.evaluate;

import org.apache.commons.lang3.StringUtils;
import weaver.conn.RecordSet;
import weaver.formmode.customjavacode.AbstractModeExpandJavaCodeNew;
import weaver.general.Util;
import weaver.hrm.resource.ResourceComInfo;
import weaver.soa.workflow.request.MainTableInfo;
import weaver.soa.workflow.request.Property;
import weaver.soa.workflow.request.RequestInfo;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 评价人员定义 添加评价人员
 *
 * @author:dxfeng
 * @createTime: 2024/05/07
 * @version: 1.0
 */
public class AddEvaluationMemberExpand extends AbstractModeExpandJavaCodeNew {
    @Override
    public Map<String, String> doModeExpand(Map<String, Object> param) {
        Map<String, String> result = new HashMap<>(2);
        RecordSet rs = new RecordSet();
        try {
            int billId;
            int modeId;
            RequestInfo requestInfo = (RequestInfo) param.get("RequestInfo");
            if (requestInfo != null) {
                billId = Util.getIntValue(requestInfo.getRequestid());
                modeId = Util.getIntValue(requestInfo.getWorkflowid());
                if (billId > 0 && modeId > 0) {
                    Map<String, String> mainDataMap = new HashMap<>(3);
                    MainTableInfo mainTableInfo = requestInfo.getMainTableInfo();
                    Property[] properties = mainTableInfo.getProperty();
                    for (Property property : properties) {
                        mainDataMap.put(property.getName(), property.getValue());
                    }

                    String bpr = mainDataMap.get("bpr");
                    if (StringUtils.isNotBlank(bpr)) {
                        ResourceComInfo resourceComInfo = new ResourceComInfo();
                        String[] bprSplit = bpr.split(",");
                        for (String s : bprSplit) {
                            String departmentId = resourceComInfo.getDepartmentID(s);
                            // 查询同部门下的所有评价人
                            rs.executeQuery("select id from hrmresource where departmentid = ? and id != ?", departmentId, s);
                            while (rs.next()) {
                                Map<String, Object> insertMap = new HashMap<>(4);
                                // 主表ID
                                insertMap.put("mainid", billId);
                                // 评价人
                                insertMap.put("pjr", rs.getString("id"));
                                // 被评人
                                insertMap.put("bpr", s);
                                // 未评价
                                insertMap.put("sfpjwc", "0");
                                // 插入数据
                                insertData(insertMap, "uf_360pjhd_dt2");
                            }
                        }
                    }


                }
            }
        } catch (Exception e) {
            rs.writeLog(e);
            result.put("errmsg", "添加评价人员拓展异常");
            result.put("flag", "false");
        }
        return result;
    }

    public static void insertData(Map<String, Object> dataMap, String tableName) {
        List<String> fieldList = new ArrayList<>();
        List<Object> dataList = new ArrayList<>();
        List<String> paramList = new ArrayList<>();

        dataMap.forEach((key, value) -> {
            if (null != value) {
                fieldList.add(key);
                dataList.add(value);
                paramList.add("?");
            }
        });
        String insertSql = " insert into " + tableName + "(" + StringUtils.join(fieldList, ",") + ") values (" + StringUtils.join(paramList, ",") + ")";
        RecordSet rs = new RecordSet();
        rs.executeUpdate(insertSql, dataList);
        if (StringUtils.isNotBlank(rs.getExceptionMsg())) {
            throw new RuntimeException(rs.getExceptionMsg());
        }
    }
}
