package weaver.interfaces.poc.action;

import com.alibaba.fastjson.JSON;
import org.apache.commons.lang3.StringUtils;
import weaver.conn.RecordSet;
import weaver.general.BaseBean;
import weaver.interfaces.workflow.action.Action;
import weaver.soa.workflow.request.*;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.List;

/**
 * @author:dxfeng
 * @createTime: 2024/05/08
 * @version: 1.0
 */
public class PerformanceDistributionAction implements Action {
    @Override
    public String execute(RequestInfo requestInfo) {
        BaseBean baseBean = new BaseBean();
        try {
            MainTableInfo mainTableInfo = requestInfo.getMainTableInfo();

            DetailTableInfo detailTableInfo = requestInfo.getDetailTableInfo();
            DetailTable detailTable0 = detailTableInfo.getDetailTable(0);

            List<Rule> ruleList = new ArrayList<>();
            Row[] rows0 = detailTable0.getRow();
            for (Row row : rows0) {
                Rule rule = new Rule();
                Cell[] cells = row.getCell();
                for (Cell cell : cells) {
                    String name = cell.getName();
                    String value = cell.getValue();
                    if ("jxdj".equals(name)) {
                        rule.setRuleName(value);
                    } else if ("fbbl".equals(name)) {
                        rule.setRuleRange(Double.parseDouble(value));
                    }

                }
                ruleList.add(rule);
            }

            ruleList.forEach(item -> baseBean.writeLog(JSON.toJSONString(item)));


            DetailTable detailTable1 = detailTableInfo.getDetailTable(1);
            String tableDBName = detailTable1.getTableDBName();
            Row[] rows1 = detailTable1.getRow();
            List<Result> resultList = new ArrayList<>();
            for (Row row : rows1) {
                Cell[] cells = row.getCell();
                Result result = new Result();
                result.setId(row.getId());
                for (Cell cell : cells) {
                    String name = cell.getName();
                    String value = cell.getValue();
                    if ("khzzdf".equals(name)) {
                        result.setScore(StringUtils.isBlank(value) ? "0" : value);
                    }
                }
                resultList.add(result);
            }

            // 所有人员排名
            LinkedHashSet<Integer> rankSet = calculateRankings(resultList);
            baseBean.writeLog("排名");
            resultList.forEach(item -> baseBean.writeLog(JSON.toJSONString(item)));

            ArrayList<Integer> rankList = new ArrayList<>(rankSet);
            // 根据排名，按照分部比例划分
            int subStart = 0;
            for (int i = 0; i < ruleList.size(); i++) {
                Rule rule = ruleList.get(i);
                double ruleRange = rule.getRuleRange();
                //int subEnd = subStart!=0?(int) Math.floor(rankList.size() * ruleRange):(int) Math.ceil(rankList.size() * ruleRange);
                // 向下取
                int subEnd = (int) Math.floor(rankList.size() * ruleRange);
                if (i == 0 && subEnd < 1) {
                    subEnd = 1;
                }
                if (i == ruleList.size() && subEnd < ruleList.size()) {
                    subEnd = rankList.size();
                }
                List<Integer> rangeList = rankList.subList(subStart, subEnd);
                for (Integer integer : rangeList) {
                    for (Result result : resultList) {
                        if (result.getRank().equals(integer)) {
                            result.setLevel(rule.getRuleName());
                        }
                    }
                }
                subStart = subEnd;
            }

            baseBean.writeLog("划分等级");

            // 更新明细表
            RecordSet rs = new RecordSet();
            String updateSql = "update " + tableDBName + " set khdj = ? where id = ?";
            for (Result result : resultList) {
                baseBean.writeLog(JSON.toJSONString(result));
                rs.executeUpdate(updateSql, result.getLevel(), result.getId());
            }


            return SUCCESS;
        } catch (Exception e) {
            baseBean.writeLog(e);
            return FAILURE_AND_CONTINUE;
        }
    }

    public static LinkedHashSet<Integer> calculateRankings(List<Result> resultList) {
        LinkedHashSet<Integer> rankSet = new LinkedHashSet<>();


        Collections.sort(resultList, (o1, o2) -> {
            return o2.getScore().compareTo(o1.getScore()); // 按值（分数）降序排序
        });

        // 计算排名并输出
        int rank = 1;
        double prevScore = -1;
        for (Result result : resultList) {
            double score = Double.parseDouble(result.getScore());
            if (score < prevScore) {
                rank++;
            }
            prevScore = score;
            result.setRank(rank);
            rankSet.add(rank);

        }

        return rankSet;
    }

    //public static LinkedHashSet<Integer> calculateRankings(List<Result> resultList) {
    //    LinkedHashSet<Integer> rankSet = new LinkedHashSet<>();
    //    int rank = 1;
    //    double prevScore = Integer.MAX_VALUE;
    //    double sameScoreCount = 0;
    //
    //    for (Result result : resultList) {
    //        double score = Double.parseDouble(result.getScore());
    //        if (score < prevScore) {
    //            rank += sameScoreCount;
    //            sameScoreCount = 1;
    //        } else {
    //            sameScoreCount++;
    //        }
    //        result.setRank(rank);
    //        rankSet.add(rank);
    //        prevScore = score;
    //    }
    //
    //    return rankSet;
    //}

    static class Rule {
        String ruleName;
        double ruleRange;

        public Rule() {

        }

        public Rule(String ruleName, double ruleRange) {
            this.ruleName = ruleName;
            this.ruleRange = ruleRange;
        }

        public String getRuleName() {
            return ruleName;
        }

        public void setRuleName(String ruleName) {
            this.ruleName = ruleName;
        }

        public double getRuleRange() {
            return ruleRange;
        }

        public void setRuleRange(double ruleRange) {
            this.ruleRange = ruleRange;
        }
    }

    static class Result {
        String id;
        String score;
        String level;
        Integer rank;

        public String getId() {
            return id;
        }

        public void setId(String id) {
            this.id = id;
        }

        public String getScore() {
            return score;
        }

        public void setScore(String score) {
            this.score = score;
        }

        public String getLevel() {
            return level;
        }

        public void setLevel(String level) {
            this.level = level;
        }

        public Integer getRank() {
            return rank;
        }

        public void setRank(Integer rank) {
            this.rank = rank;
        }
    }
}
